IMPLEMENTATION MODULE MultiSkelEdit;

FROM SYSTEM IMPORT ADR, ADDRESS;
FROM Strings IMPORT Str255, Length, Assign;
FROM ScrapManager IMPORT ZeroScrap;
FROM QuickDraw IMPORT Rect, Point, EraseRect, TextFont, TextSize;
FROM MenuManager IMPORT EnableItem, DisableItem;
FROM TextEdit IMPORT TEHandle, TEToScrap, TEFromScrap, TECut, TECopy, TEPaste,
										 TEDelete, TEActivate, TEDeactivate, TENew, TEUpdate,
										 TEDispose, TEInsert, TEKey, TEClick, TEIdle, TECalText;
FROM WindowManager IMPORT GetNewWindow, CloseWindow, WindowPtr;
FROM TransSkel IMPORT SkelWindow, WCloseProc;
FROM MultiSkelGlobals IMPORT editMenu, editWind, shiftKey, DrawGrowBox,
														 editWindRes;

CONST
	undo	= 1;
	cut		= 3;
	copy	= 4;
	paste	= 5;
	clear	= 6;

VAR
	teEdit:	TEHandle;
	whoCares: LONGINT;	(* Dummy longint *)
	soWhat: INTEGER;		(* Dummy integer *)
	

PROCEDURE EditWindHalt;

BEGIN
	TEDispose(teEdit);
	CloseWindow(editWind);
END EditWindHalt;


PROCEDURE EditWindMain;

BEGIN
	TEIdle(teEdit);
END EditWindMain;


PROCEDURE EditWindKey(ch: CHAR; mods: BITSET);

BEGIN
	TEKey(ch, teEdit);
END EditWindKey;


PROCEDURE EditWindMouse(thePt: Point; t: LONGINT; mods: BITSET);

BEGIN
	TEClick(thePt, shiftKey IN mods, teEdit);
END EditWindMouse;


(*
	Update text window.  The update event might be in response to a window
	resizing.  If so, resize the rects and recalc the lineStarts of the text.
	To resize the rects, only the right edge of the destrect need be changed
	(the bottom is not used, and the left and top remain static).  The
	viewrect should be sized to the screen.
*)

PROCEDURE EditWindUpdate(resized: BOOLEAN);

	VAR
		r: Rect;

BEGIN
	r := editWind^.portRect;
	EraseRect(r);
	INC(r.left, 4);
	DEC(r.bottom, 2);
	INC(r.top, 2);
	DEC(r.right, 19);
	
	IF resized THEN
		teEdit^^.destRect.right := r.right;
		teEdit^^.viewRect := r;
		TECalText(teEdit);
	END;
	
	DrawGrowBox(editWind);
	TEUpdate(r, teEdit);
END EditWindUpdate;


PROCEDURE EditWindActivate(active: BOOLEAN);

BEGIN
	DrawGrowBox(editWind);
	IF active THEN
		TEActivate(teEdit);
		DisableItem(editMenu, undo);
	ELSE
		TEDeactivate(teEdit);
		EnableItem(editMenu, undo);
	END;
END EditWindActivate;


(*
 *	Handle edit menu items for the text window
 *)

PROCEDURE EditWindEditMenu(item: INTEGER);

BEGIN
	CASE item OF
		(* cut selection to TEScrap, clear clipboard & put TEScrap in it *)
		cut:	TECut(teEdit);
					whoCares := ZeroScrap();
					soWhat := TEToScrap();
		|
		(* copy selection to TEScrap, clear clipboard & put TEScrap in it *)
		copy:	TECopy(teEdit);
					whoCares := ZeroScrap();
					soWhat := TEToScrap();
		|
		(* copy clipboard into TEScrap, put TEScrap into the edit record *)
		paste:	soWhat := TEFromScrap();
						TEPaste(teEdit);
		|
		(* delete selection and ignore it *)
		clear:	TEDelete(teEdit);
		ELSE (* Do nothing *)
	END;
END EditWindEditMenu;


PROCEDURE EditWindInit;

	VAR
		r: Rect;
		aStr: Str255;
		
BEGIN
	editWind := GetNewWindow(editWindRes, VAL(ADDRESS, NIL), VAL(WindowPtr, -1D));
	SkelWindow(	editWind,
							EditWindMouse,				(* Mouse click action proc *)
							EditWindKey,					(* Key click action proc *)
							EditWindUpdate,				(* Update event handler proc *)
							EditWindActivate,			(* Activate event handler proc *)
							VAL(WCloseProc, NIL),	(* No close handler proc *)
							EditWindHalt,					(* Window clobber proc *)
							EditWindMain,					(* Idle procedure *)
							TRUE);								(* Call it even if not frontmost *)
	
	TextFont(0);				(*	Display in System Font - Chicago *)
	TextSize(0);				(*	Display in System Size - 12pt *)
	
	r := editWind^.portRect;
	INC(r.left, 4);
	INC(r.top, 2);
	DEC(r.right, 19);
	DEC(r.bottom, 2);
	teEdit := TENew(r, r);
	Assign("This is a text editing window. ", aStr);
	aStr[Length(aStr)-1] := 15C;
	TEInsert(ADR(aStr), Length(aStr), teEdit);
END EditWindInit;

END MultiSkelEdit.
